--[[
-- Copyright (c) 2011-2012  Anton Savoshchanka (aka AntonioModer)
--
-- This software is provided 'as-is', without any express or implied
-- warranty.  In no event will the authors be held liable for any damages
-- arising from the use of this software.
--
-- Permission is granted to anyone to use this software for any purpose,
-- including commercial applications, and to alter it and redistribute it
-- freely, subject to the following restrictions:
--
-- 1. The origin of this software must not be misrepresented; you must not
--    claim that you wrote the original software. If you use this software
--    in a product, an acknowledgment in the product documentation would be
--    appreciated but is not required.
-- 2. Altered source versions must be plainly marked as such, and must not be
--    misrepresented as being the original software.
-- 3. This notice may not be removed or altered from any source distribution.
--]]

--	HELP:
--	create:
--		objName = newObj('className', 'objName');
--	delete:
--		objName.delObj();
--		objName = nil;
--	call function:
--		objName.someFunction();

tabClass = {};																						-- main table
tabObjCount = {};																					-- helper table
tabObjIDCounter = {};																				-- helper table

function newObj(className, objName)

	if tabClass[className] == nil then																-- init class
		tabClass[className] = {};
		tabObjCount[className] = 0;
		tabObjIDCounter[className] = 0;
		print('created '..className..' class');
	end
	
	if className == 'Entity' then																	-- init object in Entity class
		-- init obj --
		tabClass[className][objName] = {};
		local self = tabClass[className][objName];
		tabObjCount[className] = tabObjCount[className]+1;
		tabObjIDCounter[className] = tabObjIDCounter[className]+1;
		
		-- standart parameters --
		self.myName = objName;																		-- string
		self.myIndex = tabClass[className][objName];												-- table
		self.myClassName = className;																-- string		
		self.myClassIndex = tabClass[className];													-- table	
		self.myNumber = tabObjCount[className];														-- number
		self.id = tabObjIDCounter[className];														-- number
		
		
		function self.delObj()																		-- delete object
			tabClass[self.myClassName][self.myName] = nil;
			tabObjCount[self.myClassName] = tabObjCount[self.myClassName]-1;
			print('delete '..self.myClassName..':'..self.myName);
		end		
		
		-- non-standart parameters --		
		do																								
			self.tabDrawColor = {72, 160, 14, 255};												-- def green
			function self.setDrawColor(r, g, b, a)
				self.tabDrawColor[1] = r;
				self.tabDrawColor[2] = g;
				self.tabDrawColor[3] = b;
				self.tabDrawColor[4] = a;
			end
			
			-- physBodyType = "static"; "dynamic"
			-- physShapeType =  1: circle; 2: Rectangle; 3: Triangle; 4: 	
			-- TODO:
			function self.setPhysBody(x, y, physBodyType, physShapeType, rectangleWidth, rectangleHeight, tableVertices)
					
				self.x = x;                                              
				self.y = y;
				self.physShapeType = physShapeType;
				
				self.physBody = love.physics.newBody(physWorld, self.x, self.y, physBodyType);
				
				if self.physShapeType== 4 and #tableVertices == 6 then								--   - 3-
					self.physShapeType = 3
				end		
					
				if physBodyType == "dynamic" then
					self.physBody:setMass(10);
					-- self.physBody:setGravityScale(10)
					-- self.physBody:setBullet(true);												--     !!!	
				end
				
				if self.physShapeType == 1 then
					self.physShape = love.physics.newCircleShape(20);
					self.physFixture = love.physics.newFixture(self.physBody, self.physShape, 1);	
					self.physFixture:setRestitution(0);												-- bounce, def = 1	
					self.physFixture:setUserData(self.myIndex);										-- global
				end
				if self.physShapeType == 2 then
					self.physShape = love.physics.newRectangleShape(rectangleWidth, rectangleHeight);
					self.physFixture = love.physics.newFixture(self.physBody, self.physShape, 1);	
					self.physFixture:setRestitution(0);	
					self.physFixture:setUserData(self.myIndex);		
				end
				if self.physShapeType == 3 then
					self.physShapePoly = {};
					self.physFixturePoly = {};
					
					self.physShapePoly[1] = love.physics.newPolygonShape( 
						tableVertices[1], tableVertices[2], tableVertices[3], tableVertices[4], 
						tableVertices[5], tableVertices[6]);
					self.physFixturePoly[1] = love.physics.newFixture(self.physBody, self.physShapePoly[1], 1);
					self.physFixturePoly[1]:setUserData(self.myIndex);
				end
				if self.physShapeType == 4 then
					self.physShapePoly = {};
					self.physFixturePoly = {};
					
					local tableNewShape = triangulate(tableVertices);
					
					-- for tr=1, #tableNewShape do													--    (debug)
						-- print("\n".."triangle "..tr)
						-- print(tableNewShape[tr].x1, tableNewShape[tr].y1)
						-- print(tableNewShape[tr].x2, tableNewShape[tr].y2)
						-- print(tableNewShape[tr].x3, tableNewShape[tr].y3)
					-- end

					--    body   fixture --
					for tr=1, #tableNewShape do
						self.physShapePoly[tr] = love.physics.newPolygonShape( 
							tableNewShape[tr].x1, tableNewShape[tr].y1,
							tableNewShape[tr].x2, tableNewShape[tr].y2,
							tableNewShape[tr].x3, tableNewShape[tr].y3);	
						self.physFixturePoly[tr] = love.physics.newFixture(self.physBody, self.physShapePoly[tr], 1);
						self.physFixturePoly[tr]:setUserData( self.myIndex );
						
					end
				end
			end
		end
	end
	
	print('created '..className..':'..objName..' object');
	return tabClass[className][objName];
end

